﻿using System;
using Microsoft.Practices.Unity;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Infrastructure.UnitTest
{
  [TestClass]
  public class ServiceLocatorUnitTest
  {

    [ClassInitialize]
    public static void ClassInitialize(TestContext c)
    {
    // ServiceLocator.Container = new UnityContainer();
    }

    [TestMethod]
    public void ClassInitialize_Succeeds()
    {
      Assert.IsTrue(true);
    }

    [TestMethod]
    public void Constructor_WhenCalled_Succeeds()
    {
      var sl = new ServiceLocator();
      Assert.IsNotNull(sl);
    }

    [TestMethod]
    public void RegisterInstance_WhenCalled_RegistersWithUnity()
    {
      var memMgr = new TestClass();
      ServiceLocator.RegisterInstance<ITestClass>(memMgr);
      var registeredMgr = ServiceLocator.Resolve<ITestClass>();
      Assert.AreSame(memMgr, registeredMgr);
    }


    [TestMethod]
    public void RegisterInstance_WhenCalledWithName_RegistersWithUnity()
    {
      var memMgr = new TestClass();
      var name = "myname";
      ServiceLocator.RegisterInstance<ITestClass>(name, memMgr);
      var registeredMgr = ServiceLocator.Resolve<ITestClass>(name);
      Assert.AreSame(memMgr, registeredMgr);
    }


    [TestMethod]
    [ExpectedException(typeof(ResolutionFailedException))]
    public void RegisterInstance_WhenCalledWithNonExistentName_RegistersWithUnity()
    {
      var memMgr = new TestClass();
      var name = "myname";
      ServiceLocator.RegisterInstance<ITestClass>(name, memMgr);
      var registeredMgr = ServiceLocator.Resolve<ITestClass>("notfound");
      Assert.AreSame(memMgr, registeredMgr);
    }

    [TestMethod]
    public void RegisterInstance_WhenCalled_ClassAvailableFromNonGeneric()
    {
      var memMgr = new TestClass();
      ServiceLocator.RegisterInstance<ITestClass>(memMgr);
      var registeredMgr = ServiceLocator.Resolve(typeof(ITestClass));
      Assert.AreSame(memMgr, registeredMgr);
    }

    [TestMethod]
    [ExpectedException(typeof(ArgumentNullException))]
    public void GetInstance_WhenCalledAndNoContainerAssigned_ThrowsException()
    {
      ServiceLocator.Container = null;
      ServiceLocator.Resolve(typeof(ITestClass));
    }

    [TestMethod]
    [ExpectedException(typeof(ArgumentNullException))]
    public void GetInstance_WhenCalledAndTargetObjectNotRegistered_ThrowsException()
    {
      ServiceLocator.Resolve(typeof(IDoNotExist));
    }

    [TestMethod]
    [ExpectedException(typeof(ResolutionFailedException))]
    public void GetInstance_WhenCalledGenericAndTargetObjectNotRegistered_ThrowsException()
    {
      ServiceLocator.Resolve<IDoNotExist>();
    }

  }

  internal interface IDoNotExist { }
  internal interface ITestClass{}
  internal class TestClass : ITestClass{}

}
