﻿using System;

namespace Interfaces
{
	public interface IDispatchedCommand<T>
	{
		/// <summary>
		/// The key used to invoke this command
		/// </summary>
		T CommandKey { get; }

		/// <summary>
		/// Defines the method that determines whether 
		/// the command can execute in its current state.
		/// </summary>
		bool CanExecute(Object parameter);

		/// <summary>
		/// Defines the method to be called when the command is invoked.
		/// </summary>
		void Execute(Object parameter);

		/// <summary>
		/// Occurs when changes occur that affect whether 
		/// or not the command should execute.
		/// </summary>
		event EventHandler CanExecuteChanged;

		/// <summary>
		/// Number of times this command was executed.
		/// </summary>
		int ExecuteCount { get; set; }
	}
}
