﻿using System.Collections.Generic;

namespace Interfaces.Services
{
	public interface IObjectManager<T>
	{
    /// <summary>
    /// Registers a new item with the manager
    /// and returns the registered item (to enable 
    /// fluent interfaces)
    /// </summary>
		T Register(T newObject, string objectName);

    /// <summary>
    /// Returns the named item to the caller
    /// </summary>

		T Get(string pObjectName, bool pIncludeInactive = true);

    /// <summary>
    /// Returns the name of the item to the caller
    /// </summary>
		string GetName(T pObject, bool pIncludeInactive = true);

    /// <summary>
    /// Removes the first occurrence of the object from the manager
    /// </summary>
    /// <returns>
    /// True if item is successfully removed; otherwise, false. 
    /// </returns>
		bool Remove(T pObject);

    /// <summary>
    /// Removes the first occurrence of the object from the manager
    /// </summary>
    /// <returns>
    /// True if item is successfully removed; otherwise, false. 
    /// </returns>
    bool Remove(string pObjectName);

    /// <summary>
    /// Count of items being managed
    /// </summary>
    int Count { get; }

    /// <summary>
    /// Indicates whether the manager contains the given item name
    /// </summary>
    bool Contains(string pObjectName);
	}
}
