﻿using System;
using Interfaces;

namespace Services
{
	/// <summary>
	/// Base class for a command that is executed via the CommandDispatcher
	/// </summary>
	public abstract class BaseDispatchedCommand<TCommandKeyType> : IDispatchedCommand<TCommandKeyType>
	{
		public BaseDispatchedCommand(TCommandKeyType pCommandKey)
		{
			CommandKey = pCommandKey;
		}

		/// <summary>
		/// Determines whether this command can be executed.  Assume
		/// it is always available.
		/// </summary>
		public virtual bool CanExecute(object parameter)
		{
			return true;
		}

		/// <summary>
		/// Executes this command with the given parameter
		/// </summary>
		public abstract void Execute(object parameter);

		/// <summary>
		/// Notifies subscribers that the status of CanExecute has changed.
		/// </summary>
		public event EventHandler CanExecuteChanged;

		/// <summary>
		/// Key used to lookup this command.
		/// </summary>
		public TCommandKeyType CommandKey { get; internal set; }

		/// <summary>
		/// Number of times this command has been executed.
		/// </summary>
		public int ExecuteCount { get; set; }

	}

}
