﻿using System.Collections.Generic;
using System.Linq;
using Constants;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Resources;

namespace ServicesUnitTests
{
  [TestClass]
	public class ResourceProviderTest
	{
		private ResourceProvider _provider;

		[TestInitialize]
		public void Setup()
		{
			_provider = new ResourceProvider();
		}

		[TestMethod]
		public void Setup_Succeeds()
		{
			Assert.IsTrue(true);
		}

    [TestMethod]
		public void Constructor_whenCalled_CurrAssemblyIsNotNull()
		{
			Assert.IsNotNull(_provider._currAssembly);
		}

    [TestMethod]
		public void Constructor_whenCalled_IconManagerLocalVarIsNull()
		{
			//this ensures we don't create manager until we need it
			Assert.IsNull(_provider._resourceMgr);
		}

    [TestMethod]
		public void IconManager_WhenReferenced_CanBeResolved()
		{
			Assert.IsNotNull(_provider.ResourceMgr);
		}

    [TestMethod]
		public void GetIcon_WhenCalledWithBadIconName_ReturnsNull()
		{
			var str = _provider.GetResource<string>("this name does not exist");
			Assert.IsNull(str);
		}

    [TestMethod]
		public void GetIcon_WhenCalledWithValidIconName_ReturnsIcon()
		{
			var str = _provider.GetResource<string>(ResourceNames.StringResource1);
			Assert.IsFalse(string.IsNullOrEmpty(str));
		}

    [TestMethod]
		public void ResourceIcons_ForAllDefinedObject_ObjectIsReturned()
		{
			var errLst = new List<string>();
			foreach (var field in typeof(ResourceNames).GetFields())
			{
				if (_provider.GetResource<object>(field.Name) == null)
					errLst.Add(field.Name);
			}
			Assert.AreEqual(0, errLst.Count, errLst.Aggregate(
				string.Empty, (current, s) => current + (s + " ;")));
		}



	}
}
