﻿using System.Windows;
using DataRepository;
using Infrastructure;
using Interfaces.Services;
using Interfaces.ViewModels;
using Interfaces.Views;
using Microsoft.Practices.Prism.UnityExtensions;
using Microsoft.Practices.Unity;
using Services;
using ViewModels;
using Views_WPF;

namespace Shell_WPF
{

  public class BootstrapperWPF : UnityBootstrapper
  {
    private IShellFormViewModel _shellVM;

    protected override void ConfigureContainer()
    {
      base.ConfigureContainer();
      ServiceLocator.Container = Container;

      RegisterControlsAndControllers(Container);
      RegisterServices(Container);
    }


    /// <summary>
    /// Registers services for the application
    /// </summary>
    protected void RegisterServices(IUnityContainer container)
    {
      //Register this service as a singleton by using the 
      //ContainerControlledLifetimeManager parameter
      container.RegisterType<ILogger, Logger>(
        new ContainerControlledLifetimeManager());

      container.RegisterType<IApplicationSession, ApplicationSession>(
        new ContainerControlledLifetimeManager());
      container.RegisterType<IMessageAggregator, MessageAggregator>(
        new ContainerControlledLifetimeManager());
      container.RegisterType<IRepository, Repository>(
        new ContainerControlledLifetimeManager());
      container.RegisterType<ISecurityService, SecurityService>(
        new ContainerControlledLifetimeManager());
      container.RegisterType<IDialogService, DialogService>(
        new ContainerControlledLifetimeManager());

      //we are not registering the region manager by hand
      //this is done for us in the UnityBootstrapper
    }


    /// <summary>
    /// Registers the views and view models.
    /// </summary>
    protected void RegisterControlsAndControllers(IUnityContainer container)
    {
      //there should only be one navigation view so register
      //it as a singleton
      container.RegisterType<INavigationView, NavigationView>(
        new ContainerControlledLifetimeManager());
      container.RegisterType<INavigationViewModel, NavigationViewModel>(
        new ContainerControlledLifetimeManager());

      //There can be many data entry views, so use normal registration
      container.RegisterType<IDataEntryView, DataEntryView>();
      container.RegisterType<IDataEntryViewModel, DataEntryViewModel>();

      container.RegisterType<IDataGridView, DataGridView>();
      container.RegisterType<IDataGridViewModel, DataGridViewModel>();

      container.RegisterType<IStatusBarView, StatusBarView>();
      container.RegisterType<IStatusBarViewModel, StatusBarViewModel>();

      container.RegisterType<IToolbarView, ToolbarView>();
      container.RegisterType<IToolbarViewModel, ToolbarViewModel>();

      //Shell should also be a singleton
      container.RegisterType<IShellView, ShellView>(
        new ContainerControlledLifetimeManager());
      container.RegisterType<IShellFormViewModel, ShellFormViewModel>(
        new ContainerControlledLifetimeManager());
    }

    /// <summary>
    /// Creates the application shell.  This is WPF specific and should
    /// be replaced in the Winforms version.
    /// </summary>
    /// <remarks>
    /// This runs before InitializeShell
    /// </remarks>
    protected override DependencyObject CreateShell()
    {
      _shellVM = ServiceLocator.Resolve<IShellFormViewModel>();
      return _shellVM.View as DependencyObject;
    }

    /// <summary>
    /// Initializes the shell.  This is WPF specific and should be
    /// replaced in the winforms version.
    /// </summary>
    /// <remarks>
    /// This runs after CreateShell
    /// </remarks>
    protected override void InitializeShell()
    {
      base.InitializeShell();

      _shellVM.Activate();

      App.Current.MainWindow = (Window)Shell;
      App.Current.MainWindow.Show();
    }


  }
}